from AbstractScreen import AbstractScreen
import os
from sys import *
import pygame
from Loader import Loader
from State import State
from Label import Label
from Button import Button
from Sprite import Sprite
from SoundManager import SoundManager
from PhysicsManager import PhysicsManager
from pygame.locals import *
from constants import *
import util
import random
from pymunk import vec2d
from AdditiveParticle import Source
from Course import Course

class TutorialScreen(AbstractScreen):
	def __init__(self):
		AbstractScreen.__init__(self)
		self.physics = PhysicsManager()
		self.physics.setup_collisions()
		self.physics.set_callback("victory",self.on_win)
		self.physics.set_callback("sparks",self.sparks)
		self.state = State()
		self.course = False
		self.drag_start = False
		self.drag_lines = False
		self.font = Loader().load_font("slkscr",32)
		self.small_font = Loader().load_font("slkscr",16)
		self.courses = ["simple","bounce","obstacles","reverse","heart","final"]
		self.messages = [
			[
				((20,self.state.screen_size[1]/2-200),"DoubleBounce is a combination of pool and putt-putt."),
				((20,self.state.screen_size[1]/2-170),"Complete a course by hitting the white target with a scoring ball."),
				((20,self.state.screen_size[1]/2-140),"You can putt with the green cue ball."),
				((20,self.state.screen_size[1]/2-110),"Click on it, then drag in the direction of the shot.")],
			[
				((60,self.state.screen_size[1]/2-150),"You can use walls to your advantage."),
				((60,self.state.screen_size[1]/2-130),"(press 'space' to reset)")],
			[
				((200,self.state.screen_size[1]/2+150),"Sometimes the best shot isn't obvious.")],
			[
				((190,self.state.screen_size[1]/2-150),"You can also move the target by putting into it.")],
			[
				((60,self.state.screen_size[1]-50),"Shoot without thinking.")],
			[
				((60,self.state.screen_size[1]-50),"Be sure to create new courses"),
				((60,self.state.screen_size[1]-30),"and share them with the editor!")]
		]
		self.messages_on_screen = []
		self.course_index = 0
		self.in_overlay = False
		self.overlay = []
		fade = pygame.Surface(self.state.screen_size)
		fade.fill( (0,0,0) )
		fade.set_alpha(196)
		self.fade = Sprite(fade,(0,0))
		self.next = Button("Next Lesson",(0,50),32)
		self.next.pos = (self.state.screen_size[0]/2 - self.next.size[0]/2,self.state.screen_size[1]/2-self.next.size[1]/2)
		
		self.load_course(self.courses[self.course_index])
		
	def sparks(self,position,types):
		colors = [(255,255,255),(255,255,255),(64,255,64),(255,64,64),(255,255,64),(255,255,255),(64,128,255)]
		self.attach(Source(position,colors[types[0]],5))
		self.attach(Source(position,colors[types[1]],5))
		# Play glockenspiel
		if types[1] == GLASS:
			self.state.sound.play("glass")
		elif types[1] == WOOD:
			self.state.sound.play("wood")
		elif types[1] == RUBBER:
			self.state.sound.play("rubber")
		elif types[1] == TARGET:
			self.state.sound.play("target")
		elif types[1] == CUE:
			self.state.sound.play("cue")
		return True
	
	def on_win(self):
		self.in_overlay = True
		self.overlay = []
		self.overlay.append( self.fade )
		self.overlay.append( self.next )
	
	def reset(self):
		self.load_course(self.courses[self.course_index])
	
	def load_course(self,coursename):
		if self.course != False:
			self.stage.remove(self.course)
			self.course.clear()
		self.course = Course(coursename,self.physics,"tutorial")
		for message in self.messages_on_screen:
			self.stage.remove(message)
		self.messages_on_screen = []
		for message in self.messages[self.course_index]:
			widget = Label(message[1],self.small_font,(255,255,255),message[0])
			self.attach(widget)
			self.messages_on_screen.append(widget)
		self.attach(self.course)
	
	def on_focus(self):
		self.state.sound.fade_music()
		return True
	
	# Draw this screen, returning True on success
	def draw(self,screen):
		screen.fill( (0,0,0) )
		if self.drag_start != False and self.drag_lines != False:
			for line in self.drag_lines:
				pygame.draw.aaline(screen,(255,255,255),line[0],line[1],4)
		AbstractScreen.draw(self,screen)
		if self.in_overlay:
			for obj in self.overlay:
				obj.draw(screen)
		return True

	def update_drag(self,end):
		distance = (end[0] - self.drag_start.x, end[1] - self.drag_start.y)
		self.drag_lines = []
		length = util.distance(self.drag_start,vec2d.Vec2d(end))
		seglen = 16
		sx = self.drag_start.x
		sy = self.drag_start.y
		dx = distance[0]/(length/seglen + 1)
		dy = distance[1]/(length/seglen + 1)
		for i in range(0,length/seglen):
			self.drag_lines.append( ((sx,sy),(sx+dx/2,sy+dy/2)) )
			sx += dx
			sy += dy
	
	def finish_drag(self,end):
		self.course.cue.body.apply_impulse((end[0] - self.drag_start.x, end[1] - self.drag_start.y),(0,0))
		self.drag_lines = False
		self.drag_start = False
	
	def update(self):
		if not self.in_overlay:
			self.physics.update()
		AbstractScreen.update(self)
		return True

	def handle_event(self,event):
		if event.type == KEYDOWN:
			if event.key == pygame.K_ESCAPE:
				self.state.game.switch_to(2)
			elif event.key == 32: # Space
				self.reset()
		elif event.type == MOUSEBUTTONUP:
			if self.drag_start != False:
				self.finish_drag(event.pos)
		elif event.type == MOUSEMOTION:
			if self.in_overlay:
				for btn in [self.next]:
					btn.mouse_in(event.pos)
				return True
			if self.drag_start != False:
				self.update_drag(event.pos)
		elif event.type == MOUSEBUTTONDOWN:
			if self.in_overlay:
				self.in_overlay = False
				if self.next.click(event.pos):
					self.course_index += 1
					if self.course_index < len(self.courses):
						self.load_course(self.courses[self.course_index])
					else:
						self.state.game.switch_to(1)
				return True
			#print event.pos
			vec = self.course.cue.body.position
			
			if util.distance(vec,vec2d.Vec2d(event.pos)) < self.course.cue.radius+4:
				self.drag_start = vec
		return True